/*
  MCS-DROS: Monitor and Control System - Data Recorder Operating Software
  Copyright (C) 2009-2010  Virginia Tech, Christopher Wolfe <chwolfe2@vt.edu>

  This file is part of MCS-DROS.
 
  MCS-DROS is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
  
  MCS-DROS is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with MCS-DROS.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <iostream>
#include <iomanip>
#include <stdlib.h>
#include <pthread.h>
#include "TimeKeeper.h"
#include "Socket.h"
#include <list>
#include <string.h>
using namespace std;



void usage(string errorMsg){

	cout << "[Error]  " << errorMsg << endl;
	cout <<
	"Usage: " << endl <<
	"\tDataSource " << endl <<
	"\t\t-DestinationIpAddress aaa.bbb.ccc.ddd " << endl <<
	"\t\t-DestinationPort 12345 " << endl <<
	"\t\t-DataSize 1234 " << endl <<
	"\t\t-Rate 123456789012345 " << endl <<
	"\t\t-Duration 1234567890 (in milliseconds)" << endl << endl;
	exit(EXIT_FAILURE);
}
int main(int argc, char * argv[]){
	size_t totalSent=0;


	unsigned short DestinationPort=0;
	string DestinationIpAddress;
	size_t DataSize=0;
	size_t Duration=0;
	size_t Rate=0;
	bool DestinationIpAddressSpecified=false;
	bool DestinationPortSpecified=false;
	bool DataSizeSpecified=false;
	bool DurationSpecified=false;
	bool RateSpecified=false;


	int i=1;
	int p=argc-1;
	while (i<argc){
		 if (strcmp(argv[i],"-DestinationIpAddress")==0){
			if (i<p){	if (strlen(argv[i+1])<=15){		DestinationIpAddress=string(argv[i+1]);		DestinationIpAddressSpecified=true;			i++;
			} else { usage("Invalid Source length. Must not be more than 15 characters"); }		} else { usage("Missing operand: Source"); }
		 }else if (strcmp(argv[i],"-DestinationPort")==0){
		 			if (i<p){	if (strlen(argv[i+1])<=5){		DestinationPort=(unsigned short)atoi(argv[i+1]);		DestinationPortSpecified=true;			i++;
		 			} else { usage("Invalid DestinationPort length. Must not be more than 5 characters"); }		} else { usage("Missing operand: DestinationPort"); }
		 }else if (strcmp(argv[i],"-DataSize")==0){
		 			if (i<p){	if (strlen(argv[i+1])<=4){		DataSize=strtoul(argv[i+1],NULL,10);		DataSizeSpecified=true;			i++;
		 			} else { usage("Invalid DataSize length. Must not be more than 4 characters"); }		} else { usage("Missing operand: DataSize"); }
		 }else if (strcmp(argv[i],"-Duration")==0){
		 		 			if (i<p){	if (strlen(argv[i+1])<=10){		Duration=strtoul(argv[i+1],NULL,10);		DurationSpecified=true;			i++;
		 		 			} else { usage("Invalid Duration length. Must not be more than 10 characters"); }		} else { usage("Missing operand: Duration"); }
		 }else if (strcmp(argv[i],"-Rate")==0){
		 		 			if (i<p){	if (strlen(argv[i+1])<=15){		Rate=strtoul(argv[i+1],NULL,10);		RateSpecified=true;			i++;
		 		 			} else { usage("Invalid Duration length. Must not be more than 15 characters"); }		} else { usage("Missing operand: Rate"); }
		 }else {
			usage(string("Unknown commandline option:")+argv[i]);
		}
		i++;
	}


	if (
		(! DestinationIpAddressSpecified) 	||
		(! DestinationPortSpecified) 		||
		(! DataSizeSpecified) 				||
		(! RateSpecified) 					||
		(! DurationSpecified)					){
		usage("You must specify all options");
	}

	char * ptr = new char [DataSize] ;
	for(unsigned int i=0;i<DataSize;i++){
		ptr[i]=(char)(i&0xff);
	}
	size_t * ts =(size_t *) ptr;
	size_t * sid =&(((size_t *)ptr)[1]);


	Socket mySocket;
	mySocket.openOutbound(DestinationIpAddress.c_str(), DestinationPort);
	if (mySocket.outboundConnectionIsOpen() ){
		//cout << "Send socket opened Successfully." << endl;
	} else {
		cout << "Could not open send socket to "<< DestinationIpAddress << " on port " << DestinationPort << endl;
		exit (EXIT_FAILURE);
	}
	TimeKeeper::resetRuntime();
	* sid=0;
	while (
			((Duration!=0) && (TimeKeeper::getRuntime()<((double)Duration)/1000.0)) ||
			(Duration ==0)
		)

	{
		*ts=TimeKeeper::getMPM();
		totalSent+=mySocket.send(ptr,DataSize);
		(*sid)++;
		while ((((double)totalSent) /TimeKeeper::getRuntime()) > (double)Rate ){
			//wait
		}
	}

}
