#!/bin/bash
#check for DRSUs
if [ `cat /proc/partitions | grep [m][d][0-9] | wc -l` == 0 ]; then
	echo "Error: No DRSU storage units detected."
	echo "To prepare a DRSU for usage, issue the following command:"
	echo
	echo "	sudo mdadm --create /dev/md<XXX> -c 256 -l 0 -n <YYY> /dev/sd<Y0> /dev/sd<Y1> ... /dev/sd<Yn>"
	echo
	echo "  	<XXX>		is a number 0 - 5 identifying the DRSU system currently"
	echo "          		supports upto 6 atttached DRSUs in software, but only 2 in hardware.)"
	echo
	echo "  	<YYY>		is the number of component drives in the array (typically 5)"
	echo
	echo "  	<Y0> .. <Yn> 	are the block devices (drives) to be part of the array"
	echo
	echo
	echo "For Example, a typical system with a single DRSU composed of /dev/sdb through /dev/sdf:"
	echo ">sudo mdadm --create /dev/md0 -c 256 -l 0 -n 5 /dev/sdb /dev/sdc /dev/sdd /dev/sde /dev/sddf"
	echo "Device /dev/md0 started with 5 drives." 
	echo 
	echo " The system will not start without at least one DRSU present."
	echo "Now exitting."
	exit -1
fi
#check for CPU TEMP module
if [ `lsmod | grep coretemp | wc -l | tr -d "\n"` -ne 1 ]; then 
	echo "CPU temperature monitoring kernel module not found."
	echo "Inserting CPU temperature monitoring kernel module."
	modprobe coretemp; 
fi

#install network performance system variables
sysctl -e -p /LWA/config/netperformance.sysctl.conf

# main loop
CF=/LWA/config/LAUNCHOPTIONS                                                           
while [ 1 ]                                                                            
do                                                                                     
  OPTIONS=$(cat $CF)                                                             
  cd /LWA; /LWA/bin/WAVE_Y $OPTIONS                        
  RESULT=$?                                                                      
  let "FFD = ($RESULT & 0x01) == 0x01"                                           
  let "FFC = ($RESULT & 0x02) == 0x02"                                           
  let "FFS = ($RESULT & 0x04) == 0x04"                                           
  let "FFL = ($RESULT & 0x08) == 0x08"                                           
  let "SHT = ($RESULT & 0x10) == 0x10"                                           
  let "RST = ($RESULT & 0x20) == 0x20"                                           

  rm -f $CF                                                                  
  if [ $FFD == 1 ]; then echo " -flushData" >> $CF; fi                            
  if [ $FFC == 1 ]; then echo " -flushConfig" >> $CF; fi                          
  if [ $FFS == 1 ]; then echo " -flushSchedule" >> $CF; fi                        
  if [ $FFL == 1 ]; then echo " -flushLog" >> $CF; fi                             
  if [ $SHT == 1 ]; then                                                          
    if [ $RST == 1 ]; then                                                  
      shutdown -r 0;                                                 
    else                                                                   
      shutdown -h 0;                                                 
    fi                                                                     
  fi                                                                             
done 
